﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace WpfMeshValueBased
{
    /// <summary>
    /// Water drop surface generator
    /// Copyright 2016 Arction Ltd. 
    /// Usage only in LightningChart related code. 
    /// </summary>
    class WaterDropSurfaceDataGenerator
    {
        /// <summary>
        /// Oscillator properties
        /// </summary>
        private class Oscillator
        {
            public double CenterX;
            public double CenterZ;
            public double Amplitude;
            public double OffsetY;
            public double Gain;
        };


        /// <summary>
        /// Generate water drop surface data 
        /// </summary>
        /// <param name="size">Size of output array dimensions, X and Z size is same</param>
        /// <param name="xPositionsNormalized">X positions of drops</param>
        /// <param name="zPositionsNormalized">Z positions of drops</param>
        /// <param name="amplitudes">Amplitudes</param>
        /// <param name="offsetLevel">Offset level</param>
        /// <param name="volatility">Volatility, set larger number to generate more waves for each drop</param>
        /// <returns></returns>
        public static double[,] Generate(int size, double[] xPositionsNormalized,
            double[] zPositionsNormalized, double[] amplitudes, double offsetLevel, double volatility)
        {
            double[,] data = new double[size, size];
            int iOscCount = amplitudes.Length;
            Oscillator[] oscillators = new Oscillator[iOscCount];
            for (int iOsc = 0; iOsc < iOscCount; iOsc++)
            {
                oscillators[iOsc] = new Oscillator()
                {
                    Amplitude = amplitudes[iOsc],
                    CenterX = xPositionsNormalized[iOsc],
                    CenterZ = zPositionsNormalized[iOsc],
                    //Gain = amplitudes[iOsc],
                    Gain = 1,
                    OffsetY = 0,
                };

            }

            double dTotalX = 1;
            double dTotalZ = 1;
            double dStepX = (dTotalX / (double)size);
            double dStepZ = (dTotalZ / (double)size);
            double dX, dZ;

            dZ = 0;

            for (int iRow = 0; iRow < size; iRow++, dZ += dStepZ)
            {
                dX = 0;

                for (int iCol = 0; iCol < size; iCol++, dX += dStepX)
                {
                    data[iCol, iRow] = CalculateWavesAtPoint(dX, dZ, oscillators, volatility) + offsetLevel;
                }
            }

            return data;
        }


        /// <summary>
        /// Calculate waves at given point.
        /// </summary>
        /// <param name="x">wave origin x-coordinate</param>
        /// <param name="z">wave origin z-coordinate</param>
        /// <param name="oscillators">Oscillators</param>
        /// <param name="volatility">Volatility</param>
        /// <returns>wave height at given point</returns>
        private static double CalculateWavesAtPoint(double x, double z, Oscillator[] oscillators, double volatility)
        {
            double dValue = 0;
            Oscillator oscillator;
            //double dOffsetSum = 0;
            int iOscillatorCount = oscillators.Length;
            for (int i = 0; i < iOscillatorCount; i++)
            {
                oscillator = oscillators[i];

                double distX = x - oscillator.CenterX;
                double distZ = z - oscillator.CenterZ;
                double dist = Math.Sqrt(distX * distX + distZ * distZ);

                dValue += oscillator.Gain * oscillator.Amplitude * Math.Cos(dist * volatility) * Math.Exp(-dist * 3.0);

            }

            return dValue;
        }
    }
}
